from datetime import datetime
from os import path as op, listdir

from .icons import ICONS_BY_EXT, ICONS_BY_NAME


def compare(e):
    return "{0}{1}".format(isinstance(e, File), e.name)


class Entry(object):

    default_icon = 'cross.png'
    hidden = True

    def __init__(self, path, root=None):
        self.root = root
        self.path = path
        self.abspath = root.join(path)
        self.name = op.basename(self.abspath)
        self.hidden = self.name.startswith('.')

    def __new__(cls, path, root=None):
        if not root or not path or path == '/':
            return RootDirectory.__new__(RootDirectory, path)

        abspath = root.join(path)
        if op.isdir(abspath):
            return Directory.__new__(Directory, path, root)

        elif op.isfile(abspath):
            return File.__new__(File)

        raise IOError('{0} does not exists.'.format(abspath))

    def join(self, path):
        path = path.strip('/')
        return op.join(self.abspath, path) if path else self.abspath

    def parse(self, path):
        abspath = op.abspath(self.join(path))
        if self.abspath == abspath:
            return self
        return Entry(path, root=self)

    @staticmethod
    def is_file():
        return False

    @property
    def link(self):
        if not self.path.startswith('/'):
            return "/" + self.path
        return self.path

    @property
    def icon(self):
        return self.default_icon

    @property
    def modified(self):
        return datetime.fromtimestamp(op.getmtime(self.abspath))

    @property
    def size(self):
        return op.getsize(self.abspath)

    def __str__(self):
        return "<%s [%s]>" % (self.__class__.__name__, self.abspath)

    __repr__ = __str__


class Directory(Entry):
    default_icon = 'folder.png'

    def __new__(cls, *args, **kwargs):
        return object.__new__(Directory)

    @property
    def breadcrumb(self):
        paths = self.abspath[len(self.root.abspath):].split('/')
        paths.pop()
        breadcrumb = []
        while paths:
            path = op.join(*paths)
            paths.pop()
            breadcrumb.append(Entry(path, root=self.root))
        return reversed(list(enumerate(breadcrumb)))

    def is_root(self):
        return self == self.root

    @property
    def entries(self):
        entries = []

        for path in listdir(self.abspath):
            if self.hidden or not path.startswith('.'):
                entries.append(Entry(op.join(self.path, path), self.root))
        entries.sort(key=compare)

        if not self.is_root():
            entries.insert(0, ParentDirectory(self.path, self.root))

        return entries


class File(Entry):
    default_icon = 'file.png'

    def __new__(cls, *args, **kwargs):
        return object.__new__(File)

    def __init__(self, *args, **kwargs):
        super(File, self).__init__(*args, **kwargs)
        self.ext = op.splitext(self.name)[1].strip('.')

    @staticmethod
    def is_file():
        return True

    @property
    def icon(self):
        return ICONS_BY_NAME.get(
            self.name,
            ICONS_BY_EXT.get(
                self.ext,
                self.default_icon
            ))


class RootDirectory(Directory):
    default_icon = 'root.png'

    def __init__(self, path, root=None):
        self.abspath = op.abspath(path)
        super(RootDirectory, self).__init__('/', self)
        self.name = 'root'

    def __new__(cls, *args, **kwargs):
        return object.__new__(RootDirectory)

    @property
    def breadcrumb(self):
        return []

    @property
    def link(self):
        return self.path


class ParentDirectory(Directory):
    default_icon = 'parent.png'

    def __init__(self, path, root=None):
        path = op.dirname(path)
        super(ParentDirectory, self).__init__(path, root=root)
        self.name = '..'

    def __new__(cls, *args, **kwargs):
        return object.__new__(ParentDirectory)



from argparse import ArgumentParser
from os import path as op

from .app import APP
from .bottle import run
from .core import Entry


MODULE_ROOT = op.abspath(op.dirname(__file__))


def setup(path='/', port=5000, share=False, autoindex=False, hidden=False):
    APP.config.root = Entry(path)
    APP.config.static = op.join(MODULE_ROOT, 'static')
    APP.config.port = port
    APP.config.host = '127.0.0.1' if not share else '0.0.0.0'
    APP.config.autoindex = autoindex
    APP.config.hidden = hidden


def main():
    import socket

    port = 80

    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect(("127.0.0.1", 80))
        s.shutdown(2)
    except socket.error:
        port = 5000

    parser = ArgumentParser(description='Serve current directory')
    parser.add_argument('path',
                        default=op.curdir,
                        nargs='?',
                        help='Path to serve directory.')
    parser.add_argument('-p', '--port',
                        default=port,
                        help='The port of the webserver.')
    parser.add_argument('-s', '--share',
                        action='store_true',
                        help='Make server available externally.')
    parser.add_argument('-a', '--autoindex',
                        action='store_false',
                        help='Enable autoindex files.')
    parser.add_argument('-d', '--hidden',
                        action='store_false',
                        help='Hide system files.')
    args = parser.parse_args()

    setup(path=args.path, port=args.post, share=args.share,
          autoindex=args.autoindex, hidden=args.hidden)

    run(APP, port=int(APP.config.port), host=APP.config.host)


if __name__ == "__main__":
    main()



__version__ = '0.2.7'
__author__ = "Kirill Klenov <horneds@gmail.com>"
__email__ = "horneds@gmail.com"
__license__ = "GNU LGPL"
__url__ = 'http://github.com/klen/pyserve'


if __name__ == '__main__':
    from .main import main
    main()



from os.path import join, dirname

from setuptools import setup, find_packages # nolint


CURDIR = dirname(__file__)
NAME = 'pyserve'
MODULE = __import__(NAME)
README = join(CURDIR, 'README.rst')
REQUIREMENTS = open(join(CURDIR, 'requirements.txt')).readlines()
COMMANDS = ['pyserve = pyserve.main:main']


setup(
    name=NAME,
    version=MODULE.__version__,
    license=MODULE.__license__,
    author=MODULE.__author__,
    author_email=MODULE.__email__,
    url=MODULE.__url__,
    description='Serve local dirs.',
    long_description=open(README).read(),
    packages=find_packages(),
    include_package_data=True,
    install_requires=REQUIREMENTS,
    entry_points={'console_scripts': COMMANDS},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: Russian',
        'Natural Language :: English',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)', # nolint
        'Programming Language :: Python',
        'Environment :: Console',
        'Topic :: Software Development :: Code Generators',
    ],
    test_suite='tests',
    tests_require='webtest',
)
